#!/usr/bin/env python

from ctypes import *
from ctypes.util import find_library
from os import path
import sys

# For unix the prefix 'lib' is not considered.
liblinear = CDLL(path.join(path.dirname(path.abspath(__file__)), '../liblinear.so.1'))

# Construct constants
SOLVER_TYPE = ['L2R_LR', 'L2R_L2LOSS_SVC_DUAL', 'L2R_L2LOSS_SVC', 'L2R_L1LOSS_SVC_DUAL',\
		'MCSVM_CS', 'L1R_L2LOSS_SVC', 'L1R_LR', 'L2R_LR_DUAL', \
		None, None, None, \
		'L2R_L2LOSS_SVR', 'L2R_L2LOSS_SVR_DUAL', 'L2R_L1LOSS_SVR_DUAL']
for i, s in enumerate(SOLVER_TYPE): 
	if s is not None: exec("%s = %d" % (s , i))

PRINT_STRING_FUN = CFUNCTYPE(None, c_char_p)
def print_null(s): 
	return 

def genFields(names, types): 
	return list(zip(names, types))

def fillprototype(f, restype, argtypes): 
	f.restype = restype
	f.argtypes = argtypes

class feature_node(Structure):
	_names = ["index", "value"]
	_types = [c_int64, c_double]
	_fields_ = genFields(_names, _types)

	def __str__(self):
		return '%d:%g' % (self.index, self.value)

def gen_feature_nodearray(xi, feature_max=None, issparse=True):
	if isinstance(xi, dict):
		index_range = xi.keys()
	elif isinstance(xi, (list, tuple)):
		xi = [0] + xi  # idx should start from 1
		index_range = range(1, len(xi))
	else:
		raise TypeError('xi should be a dictionary, list or tuple')

	if feature_max:
		assert(isinstance(feature_max, int))
		index_range = filter(lambda j: j <= feature_max, index_range)
	if issparse: 
		index_range = filter(lambda j:xi[j] != 0, index_range)

	index_range = sorted(index_range)
	ret = (feature_node * (len(index_range)+2))()
	ret[-1].index = -1 # for bias term
	ret[-2].index = -1
	for idx, j in enumerate(index_range):
		ret[idx].index = j
		ret[idx].value = xi[j]
	max_idx = 0
	if index_range : 
		max_idx = index_range[-1]
	return ret, max_idx

class problem(Structure):
	_names = ["l", "n", "y", "x", "bias"]
	_types = [c_int64, c_int64, POINTER(c_double), POINTER(POINTER(feature_node)), c_double]
	_fields_ = genFields(_names, _types)

	def __init__(self, y, x, bias = -1):
		if len(y) != len(x) :
			raise ValueError("len(y) != len(x)")
		self.l = l = len(y)
		self.bias = -1

		max_idx = 0
		x_space = self.x_space = []
		for i, xi in enumerate(x):
			tmp_xi, tmp_idx = gen_feature_nodearray(xi)
			x_space += [tmp_xi]
			max_idx = max(max_idx, tmp_idx)
		self.n = max_idx

		self.y = (c_double * l)()
		for i, yi in enumerate(y): self.y[i] = y[i]

		self.x = (POINTER(feature_node) * l)() 
		for i, xi in enumerate(self.x_space): self.x[i] = xi

		self.set_bias(bias)

	def set_bias(self, bias):
		if self.bias == bias:
			return 
		if bias >= 0 and self.bias < 0: 
			self.n += 1
			node = feature_node(self.n, bias)
		if bias < 0 and self.bias >= 0: 
			self.n -= 1
			node = feature_node(-1, bias)

		for xi in self.x_space:
			xi[-2] = node
		self.bias = bias


class parameter(Structure):
	_names = ["solver_type", "eps", "C", "nr_weight", "weight_label", "weight", "p"]
	_types = [c_int64, c_double, c_double, c_int64, POINTER(c_int64), POINTER(c_double), c_double]
	_fields_ = genFields(_names, _types)

	def __init__(self, options = None):
		if options == None:
			options = ''
		self.parse_options(options)

	def __str__(self):
		s = ''
		attrs = parameter._names + list(self.__dict__.keys())
		values = map(lambda attr: getattr(self, attr), attrs) 
		for attr, val in zip(attrs, values):
			s += (' %s: %s\n' % (attr, val))
		s = s.strip()

		return s

	def set_to_default_values(self):
		self.solver_type = L2R_L2LOSS_SVC_DUAL
		self.eps = float('inf')
		self.C = 1
		self.p = 0.1
		self.nr_weight = 0
		self.weight_label = (c_int64 * 0)()
		self.weight = (c_double * 0)()
		self.bias = -1
		self.cross_validation = False
		self.nr_fold = 0
		self.print_func = None

	def parse_options(self, options):
		if isinstance(options, list):
			argv = options
		elif isinstance(options, str):
			argv = options.split()
		else:
			raise TypeError("arg 1 should be a list or a str.")
		self.set_to_default_values()
		self.print_func = cast(None, PRINT_STRING_FUN)
		weight_label = []
		weight = []

		i = 0
		while i < len(argv) :
			if argv[i] == "-s":
				i = i + 1
				self.solver_type = int(argv[i])
			elif argv[i] == "-c":
				i = i + 1
				self.C = float(argv[i])
			elif argv[i] == "-p":
				i = i + 1
				self.p = float(argv[i])
			elif argv[i] == "-e":
				i = i + 1
				self.eps = float(argv[i])
			elif argv[i] == "-B":
				i = i + 1
				self.bias = float(argv[i])
			elif argv[i] == "-v":
				i = i + 1
				self.cross_validation = 1
				self.nr_fold = int(argv[i])
				if self.nr_fold < 2 :
					raise ValueError("n-fold cross validation: n must >= 2")
			elif argv[i].startswith("-w"):
				i = i + 1
				self.nr_weight += 1
				nr_weight = self.nr_weight
				weight_label += [int(argv[i-1][2:])]
				weight += [float(argv[i])]
			elif argv[i] == "-q":
				self.print_func = PRINT_STRING_FUN(print_null)
			else :
				raise ValueError("Wrong options")
			i += 1

		liblinear.set_print_string_function(self.print_func)
		self.weight_label = (c_int64*self.nr_weight)()
		self.weight = (c_double*self.nr_weight)()
		for i in range(self.nr_weight): 
			self.weight[i] = weight[i]
			self.weight_label[i] = weight_label[i]

		if self.eps == float('inf'):
			if self.solver_type in [L2R_LR, L2R_L2LOSS_SVC]:
				self.eps = 0.01
			elif self.solver_type in [L2R_L2LOSS_SVR]:
				self.eps = 0.001
			elif self.solver_type in [L2R_L2LOSS_SVC_DUAL, L2R_L1LOSS_SVC_DUAL, MCSVM_CS, L2R_LR_DUAL]:
				self.eps = 0.1
			elif self.solver_type in [L1R_L2LOSS_SVC, L1R_LR]:
				self.eps = 0.01
			elif self.solver_type in [L2R_L2LOSS_SVR_DUAL, L2R_L1LOSS_SVR_DUAL]:
				self.eps = 0.1

class model(Structure):
	_names = ["param", "nr_class", "nr_feature", "w", "label", "bias"]
	_types = [parameter, c_int64, c_int64, POINTER(c_double), POINTER(c_int64), c_double]
	_fields_ = genFields(_names, _types)

	def __init__(self):
		self.__createfrom__ = 'python'

	def __del__(self):
		# free memory created by C to avoid memory leak
		if hasattr(self, '__createfrom__') and self.__createfrom__ == 'C':
			liblinear.free_and_destroy_model(pointer(self))

	def get_nr_feature(self):
		return liblinear.get_nr_feature(self)

	def get_nr_class(self):
		return liblinear.get_nr_class(self)

	def get_labels(self):
		nr_class = self.get_nr_class()
		labels = (c_int64 * nr_class)()
		liblinear.get_labels(self, labels)
		return labels[:nr_class]

	def is_probability_model(self):
		return (liblinear.check_probability_model(self) == 1)

def toPyModel(model_ptr):
	"""
	toPyModel(model_ptr) -> model

	Convert a ctypes POINTER(model) to a Python model
	"""
	if bool(model_ptr) == False:
		raise ValueError("Null pointer")
	m = model_ptr.contents
	m.__createfrom__ = 'C'
	return m

fillprototype(liblinear.train, POINTER(model), [POINTER(problem), POINTER(parameter)])
fillprototype(liblinear.cross_validation, None, [POINTER(problem), POINTER(parameter), c_int64, POINTER(c_double)])

fillprototype(liblinear.predict_values, c_double, [POINTER(model), POINTER(feature_node), POINTER(c_double)])
fillprototype(liblinear.predict, c_double, [POINTER(model), POINTER(feature_node)])
fillprototype(liblinear.predict_probability, c_double, [POINTER(model), POINTER(feature_node), POINTER(c_double)])

fillprototype(liblinear.save_model, c_int64, [c_char_p, POINTER(model)])
fillprototype(liblinear.load_model, POINTER(model), [c_char_p])

fillprototype(liblinear.get_nr_feature, c_int64, [POINTER(model)])
fillprototype(liblinear.get_nr_class, c_int64, [POINTER(model)])
fillprototype(liblinear.get_labels, None, [POINTER(model), POINTER(c_int64)])

fillprototype(liblinear.free_model_content, None, [POINTER(model)])
fillprototype(liblinear.free_and_destroy_model, None, [POINTER(POINTER(model))])
fillprototype(liblinear.destroy_param, None, [POINTER(parameter)])
fillprototype(liblinear.check_parameter, c_char_p, [POINTER(problem), POINTER(parameter)])
fillprototype(liblinear.check_probability_model, c_int64, [POINTER(model)])
fillprototype(liblinear.set_print_string_function, None, [CFUNCTYPE(None, c_char_p)])



from collections import defaultdict
import cPickle
import os

import jieba
from base import *

__all__ = ['GroceryTextConverter']


def _dict2list(d):
    if len(d) == 0:
        return []
    m = max(v for k, v in d.iteritems())
    ret = [''] * (m + 1)
    for k, v in d.iteritems():
        ret[v] = k
    return ret


def _list2dict(l):
    return dict((v, k) for k, v in enumerate(l))


class GroceryTextPreProcessor(object):
    def __init__(self):
        # index must start from 1
        self.tok2idx = {'>>dummy<<': 0}
        self.idx2tok = None

    @staticmethod
    def _default_tokenize(text):
        return jieba.cut(text, cut_all=True)

    def preprocess(self, text, custom_tokenize):
        if custom_tokenize is not None:
            tokens = custom_tokenize(text)
        else:
            tokens = self._default_tokenize(text)
        ret = []
        for idx, tok in enumerate(tokens):
            if tok not in self.tok2idx:
                self.tok2idx[tok] = len(self.tok2idx)
            ret.append(self.tok2idx[tok])
        return ret

    def save(self, dest_file):
        self.idx2tok = _dict2list(self.tok2idx)
        config = {'idx2tok': self.idx2tok}
        cPickle.dump(config, open(dest_file, 'wb'), -1)

    def load(self, src_file):
        config = cPickle.load(open(src_file, 'rb'))
        self.idx2tok = config['idx2tok']
        self.tok2idx = _list2dict(self.idx2tok)
        return self


class GroceryFeatureGenerator(object):
    def __init__(self):
        self.ngram2fidx = {'>>dummy<<': 0}
        self.fidx2ngram = None

    def unigram(self, tokens):
        feat = defaultdict(int)
        NG = self.ngram2fidx
        for x in tokens:
            if (x,) not in NG:
                NG[x,] = len(NG)
            feat[NG[x,]] += 1
        return feat

    def bigram(self, tokens):
        feat = self.unigram(tokens)
        NG = self.ngram2fidx
        for x, y in zip(tokens[:-1], tokens[1:]):
            if (x, y) not in NG:
                NG[x, y] = len(NG)
            feat[NG[x, y]] += 1
        return feat

    def save(self, dest_file):
        self.fidx2ngram = _dict2list(self.ngram2fidx)
        config = {'fidx2ngram': self.fidx2ngram}
        cPickle.dump(config, open(dest_file, 'wb'), -1)

    def load(self, src_file):
        config = cPickle.load(open(src_file, 'rb'))
        self.fidx2ngram = config['fidx2ngram']
        self.ngram2fidx = _list2dict(self.fidx2ngram)
        return self


class GroceryClassMapping(object):
    def __init__(self):
        self.class2idx = {}
        self.idx2class = None

    def to_idx(self, class_name):
        if class_name in self.class2idx:
            return self.class2idx[class_name]

        m = len(self.class2idx)
        self.class2idx[class_name] = m
        return m

    def to_class_name(self, idx):
        if self.idx2class is None:
            self.idx2class = _dict2list(self.class2idx)
        if idx == -1:
            return "**not in training**"
        if idx >= len(self.idx2class):
            raise KeyError(
                'class idx ({0}) should be less than the number of classes ({0}).'.format(idx, len(self.idx2class)))
        return self.idx2class[idx]

    def save(self, dest_file):
        self.idx2class = _dict2list(self.class2idx)
        config = {'idx2class': self.idx2class}
        cPickle.dump(config, open(dest_file, 'wb'), -1)

    def load(self, src_file):
        config = cPickle.load(open(src_file, 'rb'))
        self.idx2class = config['idx2class']
        self.class2idx = _list2dict(self.idx2class)
        return self


class GroceryTextConverter(object):
    def __init__(self, custom_tokenize=None):
        self.text_prep = GroceryTextPreProcessor()
        self.feat_gen = GroceryFeatureGenerator()
        self.class_map = GroceryClassMapping()
        self.custom_tokenize = custom_tokenize

    def get_class_idx(self, class_name):
        return self.class_map.to_idx(class_name)

    def get_class_name(self, class_idx):
        return self.class_map.to_class_name(class_idx)

    def to_svm(self, text, class_name=None):
        feat = self.feat_gen.bigram(self.text_prep.preprocess(text, self.custom_tokenize))
        if class_name is None:
            return feat
        return feat, self.class_map.to_idx(class_name)

    def convert_text(self, text_src, delimiter, output=None):
        if not output:
            output = '%s.svm' % text_src
        text_src = read_text_src(text_src, delimiter)
        with open(output, 'w') as w:
            for line in text_src:
                try:
                    label, text = line
                except ValueError:
                    continue
                feat, label = self.to_svm(text, label)
                w.write('%s %s\n' % (label, ''.join(' {0}:{1}'.format(f, feat[f]) for f in sorted(feat))))

    def save(self, dest_dir):
        config = {
            'text_prep': 'text_prep.config.pickle',
            'feat_gen': 'feat_gen.config.pickle',
            'class_map': 'class_map.config.pickle',
        }
        if not os.path.exists(dest_dir):
            os.mkdir(dest_dir)
        self.text_prep.save(os.path.join(dest_dir, config['text_prep']))
        self.feat_gen.save(os.path.join(dest_dir, config['feat_gen']))
        self.class_map.save(os.path.join(dest_dir, config['class_map']))

    def load(self, src_dir):
        config = {
            'text_prep': 'text_prep.config.pickle',
            'feat_gen': 'feat_gen.config.pickle',
            'class_map': 'class_map.config.pickle',
        }
        self.text_prep.load(os.path.join(src_dir, config['text_prep']))
        self.feat_gen.load(os.path.join(src_dir, config['feat_gen']))
        self.class_map.load(os.path.join(src_dir, config['class_map']))
        return self



import uuid
import os
import shutil

from converter import GroceryTextConverter
from .learner import *
from base import *


class GroceryTextModel(object):
    def __init__(self, text_converter=None, model=None):
        if isinstance(text_converter, GroceryTextConverter):
            self.text_converter = text_converter
        self.svm_model = model
        self._hashcode = str(uuid.uuid4())

    def __str__(self):
        return 'TextModel instance ({0}, {1})'.format(self.text_converter, self.svm_model)

    def get_labels(self):
        return [self.text_converter.get_class_name(k) for k in self.svm_model.get_labels()]

    def load(self, model_name):
        try:
            with open(model_name + '/id', 'r') as fin:
                self._hashcode = fin.readline().strip()
        except IOError:
            raise ValueError("The given model is invalid.")
        self.text_converter = GroceryTextConverter().load(model_name + '/converter')
        self.svm_model = LearnerModel(model_name + '/learner')

    def save(self, model_name, force=False):
        if self.svm_model is None:
            raise Exception('This model can not be saved because svm model is not given.')
        if os.path.exists(model_name) and force:
            shutil.rmtree(model_name)
        try:
            os.mkdir(model_name)
        except OSError as e:
            raise OSError(e, 'Please use force option to overwrite the existing files.')
        self.text_converter.save(model_name + '/converter')
        self.svm_model.save(model_name + '/learner', force)

        with open(model_name + '/id', 'w') as fout:
            fout.write(self._hashcode)

    def predict_text(self, text):
        if self.svm_model is None:
            raise Exception('This model is not usable because svm model is not given')
        # process unicode type
        if isinstance(text, unicode):
            text = text.encode('utf-8')
        if not isinstance(text, str):
            raise TypeError('The argument should be plain text')
        text = self.text_converter.to_svm(text)
        y, dec = predict_one(text, self.svm_model)
        y = self.text_converter.get_class_name(int(y))
        labels = [self.text_converter.get_class_name(k) for k in
                  self.svm_model.label[:self.svm_model.nr_class]]
        return GroceryPredictResult(predicted_y=y, dec_values=dec[:self.svm_model.nr_class], labels=labels)


class GroceryTest(object):
    def __init__(self, model):
        self.model = model

    def test(self, text_src, delimiter):
        text_src = read_text_src(text_src, delimiter)
        true_y = []
        predicted_y = []
        for line in text_src:
            try:
                label, text = line
            except ValueError:
                continue
            predicted_y.append(self.model.predict_text(text).predicted_y)
            true_y.append(label)
        return GroceryTestResult(true_y, predicted_y)



def read_text_src(text_src, delimiter):
    if isinstance(text_src, str):
        with open(text_src, 'r') as f:
            text_src = [line.split(delimiter) for line in f]
    elif not isinstance(text_src, list):
        raise TypeError('text_src should be list or str')
    return text_src


class GroceryTestResult(object):
    def __init__(self, true_y, predicted_y):
        self.true_y = true_y
        self.predicted_y = predicted_y
        self._compute_accuracy_overall()
        self._compute_accuracy_recall_labels()

    def _compute_accuracy_overall(self):
        l = len(self.true_y)
        self.accuracy_overall = sum([self.true_y[i] == self.predicted_y[i] for i in range(l)]) / float(l)

    def _compute_accuracy_recall_labels(self):
        labels = {}
        for idx, predicted_label in enumerate(self.predicted_y):
            true_label = self.true_y[idx]
            if predicted_label not in labels:
                labels[predicted_label] = [0, 0, 0]
            if true_label not in labels:
                labels[true_label] = [0, 0, 0]
            if predicted_label == true_label:
                labels[predicted_label][0] += 1
            labels[predicted_label][1] += 1
            labels[true_label][2] += 1
        self.accuracy_labels = {}
        self.recall_labels = {}
        for key, val in labels.iteritems():
            try:
                self.accuracy_labels[key] = float(val[0]) / val[1]
            except ZeroDivisionError:
                self.accuracy_labels[key] = float(0)
            try:
                self.recall_labels[key] = float(val[0]) / val[2]
            except ZeroDivisionError:
                self.recall_labels[key] = float(0)

    @staticmethod
    def draw_table(data, row_labels, column_labels):
        row_format = '{:<15}' * (len(column_labels) + 1)
        table_string = '%s\n' % row_format.format('', *column_labels)
        for row_label, row_data in zip(row_labels, data):
            table_string += '%s\n' % row_format.format(row_label, *row_data)
        return table_string

    def show_result(self):
        print self.draw_table(
            zip(
                ['%.2f%%' % (s * 100) for s in self.accuracy_labels.values()],
                ['%.2f%%' % (s * 100) for s in self.recall_labels.values()]
            ),
            self.accuracy_labels.keys(),
            ('accuracy', 'recall')
        )

    def __str__(self):
        return str(self.accuracy_overall)


class GroceryPredictResult(object):
    def __init__(self, predicted_y=None, dec_values=None, labels=None):
        self.predicted_y = predicted_y
        self.dec_values = dict(zip(labels, dec_values))

    def __str__(self):
        return self.predicted_y



from converter import *
from classifier import *


__all__ = ['Grocery']


class GroceryException(Exception):
    pass


class GroceryNotTrainException(GroceryException):
    def __init__(self):
        self.message = 'Text model has not been trained.'


class Grocery(object):
    def __init__(self, name, custom_tokenize=None):
        self.name = name
        if custom_tokenize is not None:
            if not hasattr(custom_tokenize, '__call__'):
                raise GroceryException('Tokenize func must be callable.')
        self.custom_tokenize = custom_tokenize
        self.model = None
        self.classifier = None
        self.train_svm_file = None

    def get_load_status(self):
        return self.model is not None and isinstance(self.model, GroceryTextModel)

    def train(self, train_src, delimiter='\t'):
        text_converter = GroceryTextConverter(custom_tokenize=self.custom_tokenize)
        self.train_svm_file = '%s_train.svm' % self.name
        text_converter.convert_text(train_src, output=self.train_svm_file, delimiter=delimiter)
        # default parameter
        model = train(self.train_svm_file, '', '-s 4')
        self.model = GroceryTextModel(text_converter, model)
        return self

    def predict(self, single_text):
        if not self.get_load_status():
            raise GroceryNotTrainException()
        return self.model.predict_text(single_text)

    def test(self, text_src, delimiter='\t'):
        if not self.get_load_status():
            raise GroceryNotTrainException()
        return GroceryTest(self.model).test(text_src, delimiter)

    def save(self):
        if not self.get_load_status():
            raise GroceryNotTrainException()
        self.model.save(self.name, force=True)

    def load(self):
        self.model = GroceryTextModel(self.custom_tokenize)
        self.model.load(self.name)

    def __del__(self):
        if self.train_svm_file and os.path.exists(self.train_svm_file):
            os.remove(self.train_svm_file)


#!/usr/bin/env python

from ctypes import *
from ctypes.util import find_library
import sys
import os
from os import path
import shutil

if sys.version_info[0] >= 3:
    xrange = range
    import pickle as cPickle
    izip = zip

    def unicode(string, setting):
        return string
else:
    import cPickle
    from itertools import izip

util = CDLL(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'util.so.1'))

LIBLINEAR_HOME = os.environ.get('LIBLINEAR_HOME') or os.path.dirname(os.path.abspath(__file__)) + '/liblinear'
sys.path = [LIBLINEAR_HOME, LIBLINEAR_HOME + '/python'] + sys.path

import liblinear
from liblinearutil import train as liblinear_train, predict as liblinear_predict, save_model as liblinear_save_model, load_model as liblinear_load_model

__all__ = ['LearnerParameter', 'LearnerModel',
        'train', 'predict_one', 'predict', 'LIBLINEAR_HOME']


def print_debug(src):
    if os.environ.get('SHORTTEXTDEBUG'):
        print('[DEBUG]: ' + src)


def fillprototype(f, restype, argtypes):
    f.restype = restype
    f.argtypes = argtypes


def genFields(names, types):
    return list(zip(names, types))


# Interface to util
class SVMProblem(Structure):
    _names = ["prob", "x_space", "n_x_space"]
    _types = [liblinear.problem, POINTER(liblinear.feature_node), c_int64]
    _fields_ = genFields(_names, _types)

    def __del__ (self):
        print_debug('SVMProblem delete:%s'% id(self))
        util.freeSVMProblem(self)


def read_SVMProblem(src):
    status = c_int64()
    svmprob = util.read_problem(src.encode(), 0, pointer(status))  # bias = 0 is required

    status = status.value

    if status == 0:
        print_debug('SVMProblem construct:%s'% id(svmprob))
        return svmprob

    if status == -1:
        raise IOError("Can not open file " + src + ".")

    if status == -2:
        raise MemoryError("Memory Exhausted. Try to restart python.")

    raise ValueError("Wrong file format in line " + str(status) + ".")


fillprototype(util.read_problem, SVMProblem, [c_char_p, c_double, POINTER(c_int64)])
fillprototype(util.freeSVMProblem, None, [SVMProblem])
fillprototype(util.compute_idf, c_double, [POINTER(liblinear.problem), POINTER(c_double)])
fillprototype(util.normalize, None, [POINTER(liblinear.problem), c_int, c_int, c_int, c_int, POINTER(c_double)])


class LearnerProblem(liblinear.problem):
    def __init__(self, src):
        #svmprob = util.read_problem(src.encode(), 0)  # bias = 0 is required
        svmprob = read_SVMProblem(src)  # bias = 0 is required
        self.x = svmprob.prob.x
        self.y = svmprob.prob.y
        self.l = svmprob.prob.l
        self.n = svmprob.prob.n
        self.bias = svmprob.prob.bias
        self.x_space = svmprob.x_space
        self.n_x_space = svmprob.n_x_space
        print_debug('LearnerProblem construct:%s'% id(svmprob))

    def set_bias(self, bias):
        if self.bias == bias:
            return
        node = liblinear.feature_node(self.n, bias)
        if bias >= 0 and self.bias < 0:
            self.n += 1
            node = liblinear.feature_node(self.n, bias)
        if bias < 0 and self.bias >= 0:
            self.n -= 1
            node = liblinear.feature_node(-1, bias)

        for i in range(1,self.l):
            self.x[i][-2] = node
        self.x_space[self.n_x_space-2] = node
        self.bias = bias

    def normalize(self, learner_param, idf):
        print_debug ("normal parameters: bin_feat {0}, inst_norm {1}, tf {2}, idf {3}\n".format(learner_param.binary_feature,
            learner_param.inst_normalization,
            learner_param.term_frequency,
            learner_param.inverse_document_frequency,
        ))
        util.normalize(pointer(self),
            learner_param.binary_feature,
            learner_param.inst_normalization,
            learner_param.term_frequency,
            learner_param.inverse_document_frequency,
            idf)

    @staticmethod
    def normalize_one(xi, learner_param, idf):
        """
        The maximum index of xi should be less
        or equal to the weight vector size.
        """
        norm = 0
        word_count = 0
        i = 0
        while xi[i].index != -1:
            idx = xi[i].index-1
            if learner_param.binary_feature:
                xi[i].value = xi[i].value != 0

            word_count += abs(xi[i].value)

            if learner_param.inverse_document_frequency and idx < len(idf):
                xi[i].value *= idf[idx]

            norm += xi[i].value * xi[i].value
            i += 1

        norm **= .5


        if learner_param.term_frequency:
            i = 0
            while xi[i].index != -1:
                xi[i].value /= word_count
                i += 1

        if learner_param.inst_normalization:
            i = 0
            while xi[i].index != -1:
                xi[i].value /= norm
                i += 1

    def compute_idf(self):
        idf = (c_double * self.n)()
        util.compute_idf(self, idf)
        return idf

class LearnerParameter(liblinear.parameter):
    """
    :class:`LearnerParameter` is the parameter structure used by
    :class:`LearnerModel`. It consists of normalization parameters and
    LIBLINEAR parameters.

    Both *liblinear_opts* and *learner_opts* are :class:`str` or a
    :class:`list` of :class:`str`. For example, you can write either

    >>> param = LearnerParameter('-N 1 -T 1', '-c 2 -e 1e-2')

    or

    >>> param = LearnerParameter(['-N', '1', '-T', '1'], ['-c', '2', '-e', '1e-2'])

    *liblinear_opts* is LIBLINEAR's parameters. Refer to LIBLINEAR's
    document for more details. *learner_opts* includes options for feature
    representation and instance-wise normalization. The preprocessor of
    LibShortText converts text files to LIBSVM-format data, where the
    features are word counts. All *value* in the options should be either
    ``1`` or ``0``, where ``1`` enables the option.

        ========== ====================================================
        options    explanation when *value* is ``1``
        ========== ====================================================
        -D *value* Binary representation. All non-zero values are
                   treated as 1. Default is enabled.
        -T *value* Term frequency. The data are divided by the feature
                   sum. That is,
                   :math:`x_i \leftarrow (x_i)/\sum_j |x_j|`,
                   where :math:`x` is the training instance and
                   :math:`x_i` is the :math:`i`-th feature of :math:`x`.
                   Default is disabled.
        -I *value* Inverse document frequency (idf). Default is
                   disabled.
        -N *value* Instance normalization. The training instances are
                   normalized to unit vectors before training. Default
                   is enabled.
        ========== ====================================================

    Note that if more than one option is enabled, then they are done in the
    order: binary representation, term frequency, IDF, and instance
    normalization. The following example is tf-idf representation without
    instance normalization.

    >>> param = LearnerParameter('-D 0 -T 1 -I 1 -N 0', liblinear_opts)

    """
    def __init__(self, learner_opts = '', liblinear_opts = ''):
        self.parse_options(learner_opts, liblinear_opts)

    def set_to_default_values(self):
        """
        Set the options to some values
        (``'-D 1 -T 0 -I 0 -N 1'``).
        """
        liblinear.parameter.set_to_default_values(self)
        self.binary_feature = 1
        self.inst_normalization = 1
        self.term_frequency = 0
        self.inverse_document_frequency = 0

    def parse_options(self, learner_opts, liblinear_opts):
        """
        Set the options to the specific values.
        """

        self.raw_options = (learner_opts, liblinear_opts)
        if isinstance(learner_opts, list):
            argv = learner_opts
        elif isinstance(learner_opts, str):
            argv = learner_opts.split()
        else:
            raise TypeError("Wrong types")
        self.set_to_default_values()
        liblinear.parameter.parse_options(self, liblinear_opts)

        i = 0
        while i < len(argv):
            if argv[i] == "-D":
                i = i + 1
                self.binary_feature = int(argv[i])
            elif argv[i] == "-N":
                i = i + 1
                self.inst_normalization = int(argv[i])
            elif argv[i] == "-I":
                i = i + 1
                self.inverse_document_frequency = int(argv[i])
            elif argv[i] == "-T":
                i = i + 1
                self.term_frequency = int(argv[i])
            else :
                raise ValueError('No option ' + argv[i])
            i = i + 1


class LearnerModel(liblinear.model):
    """
    :class:`LearnerModel` is a middle-level classification model. It
    inherits from :class:`liblinear.model` by having two more members:
    a :class:`LearnerParameter` instance and an inverse document frequency list.

    We do not recommend users to create a :class:`LearnerModel` by themselves.
    Instead, users should create and manipulate a :class:`LearnerModel`
    via :func:`train`, :func:`predict`, and :func:`predict_one`.

    If users want to redefine :class:`LearnerModel`, they must
    implement the following four methods used by
    :mod:`libshorttext.classifier` and :mod:`libshorttext.analyzer`.
    """

    def _reconstruct_label_idx(self):
        def _get_label_idx(nr_class, labels):
            return dict(zip(labels[:nr_class], range(nr_class)))

        if self.c_model is not None:
            self.labelidx = _get_label_idx(self.c_model.nr_class, self.c_model.label)


    def __init__(self, c_model, param = None, idf = None):
        """
        constructor of :class:`LearnerModel`.
        """

        print_debug('c_model(%s), self(%s)' % (id(c_model), id(self)))

        if isinstance(c_model, str):
            self.load(c_model)
            return
        elif isinstance(c_model, liblinear.model):
            if param is None:
                raise ValueError("param can not be None if model is given.")
        else:
            raise TypeError("c_model should be model file name or a model.")

        self.c_model = c_model # prevent GC

        if isinstance(param, LearnerParameter):
            self.param_options = param.raw_options
        elif isinstance(param, tuple):
            self.param_options = param
        else:
            raise TypeError("param should be a LearnerParameter or a tuple.")

        if idf is not None:
            self.idf = idf[:self.c_model.nr_feature + (self.c_model.bias >= 0)]
        else:
            self.idf = None

        for attr in c_model._names:
            setattr(self, attr, getattr(c_model, attr))

        self._reconstruct_label_idx()

    def get_weight(self, j, k):
        """
        Return the weight of feature *j* and label *k*.
        """
        return self.c_model.w[(j-1)*self.c_model.nr_class + self.labelidx[k]]

    def get_labels(self):
        """
        Return the labels of this model.
        """
        return self.label[:self.nr_class]

    def load(self, model_dir):
        """
        Load the contents from a :class:`TextModel` directory.
        """

        self.c_model = liblinear_load_model(path.join(model_dir,'liblinear_model'))

        options_file = path.join(model_dir,'options.pickle')
        self.param_options = cPickle.load(open(options_file,'rb'))

        idf_file = path.join(model_dir,'idf.pickle')
        self.idf = cPickle.load(open(idf_file,'rb'))

        self.__init__(self.c_model, LearnerParameter(self.param_options[0], self.param_options[1]), self.idf)

    def save(self, model_dir, force=False):
        """
        Save the model to a directory. If *force* is set to ``True``,
        the existing directory will be overwritten; otherwise,
        :class:`IOError` will be raised.
        """

        if path.exists(model_dir):
            if force:
                shutil.rmtree(model_dir)
            else :
                raise OSError('Please use force option to overwrite the existing files.')
        os.mkdir(model_dir)

        liblinear_save_model(path.join(model_dir,'liblinear_model'), self.c_model)
        options_file = path.join(model_dir,'options.pickle')
        cPickle.dump(self.param_options, open(options_file,'wb'),-1)

        idf_file = path.join(model_dir,'idf.pickle')
        cPickle.dump(self.idf, open(idf_file,'wb'),-1)

    def __str__(self):
        if type(self.param_options) is tuple and len(self.param_options) > 0:
            return 'LearnerModel: ' + (self.param_options[0] or 'default')
        else:
            return 'empty LearnerModel'


def train(data_file_name, learner_opts="", liblinear_opts=""):
    """
    Return a :class:`LearnerModel`.

    *data_file_name* is the file path of the LIBSVM-format data. *learner_opts* is a
    :class:`str`. Refer to :ref:`learner_param`. *liblinear_opts* is a :class:`str` of
    LIBLINEAR's parameters. Refer to LIBLINEAR's document.
    """

    learner_prob = LearnerProblem(data_file_name)
    learner_param = LearnerParameter(learner_opts, liblinear_opts)

    idf = None
    if learner_param.inverse_document_frequency:
        idf = learner_prob.compute_idf()

    learner_prob.normalize(learner_param, idf)

    m = liblinear_train(learner_prob, learner_param)
    if not learner_param.cross_validation:
        m.x_space = None  # This is required to reduce the memory usage...
        m = LearnerModel(m, learner_param, idf)
    return m


def predict_one(xi, m):
    """
    Return the label and a :class:`c_double` array of decision values of
    the test instance *xi* using :class:`LearnerModel` *m*.

    *xi* can be a :class:`list` or a :class:`dict` as in LIBLINEAR python
    interface. It can also be a LIBLINEAR feature_node array.

    .. note::

        This function is designed to analyze the result of one instance.
        It has a severe efficiency issue and should be used only by
        :func:`libshorttext.classifier.predict_single_text`. If many
        instances need to be predicted, they should be stored in a file
        and predicted by :func:`predict`.

    .. warning::

        The content of *xi* may be **changed** after the function call.
    """

    if isinstance(xi, (list, dict)):
        xi = liblinear.gen_feature_nodearray(xi)[0]
    elif not isinstance(xi, POINTER(liblinear.feature_node)):
        raise TypeError("xi should be a test instance")

    learner_param = LearnerParameter(m.param_options[0], m.param_options[1])

    if m.bias >= 0:
        i = 0
        while xi[i].index != -1: i += 1

        # Already has bias, or bias reserved.
        # Actually this statement should be true if
        # the data is read by read_SVMProblem.
        if i > 0 and xi[i-1].index == m.nr_feature + 1:
            i -= 1

        xi[i] = liblinear.feature_node(m.nr_feature + 1, m.bias)
        xi[i+1] = liblinear.feature_node(-1, 0)

    LearnerProblem.normalize_one(xi, learner_param, m.idf)

    dec_values = (c_double * m.nr_class)()
    label = liblinear.liblinear.predict_values(m, xi, dec_values)

    return label, dec_values


def predict(data_file_name, m, liblinear_opts=""):
    """
    Return a quadruple: the predicted labels, the accuracy, the decision values, and the
    true labels in the test data file (obtained through the :class:`LearnerModel` *m*).

    The predicted labels and true labels in the file are :class:`list`. The accuracy is
    evaluated by assuming that the labels in the file are the true label.

    The decision values are in a :class:`list`, where the length is the same as the number
    of test instances. Each element in the list is a :class:`c_double` array, and the
    values in the array are an instance's decision values in different classes.
    For example, the decision value of instance i and class k can be obtained by

    >>> predicted_label, accuracy, all_dec_values, label = predict('svm_file', model)
    >>> print all_dec_values[i][k]
    """

    learner_prob = LearnerProblem(data_file_name)
    learner_param = LearnerParameter(m.param_options[0], m.param_options[1])

    idf = None
    if m.idf:
        idf = (c_double * len(m.idf))()
        for i in range(len(m.idf)): idf[i] = m.idf[i]
    learner_prob.normalize(learner_param, idf)

    all_dec_values = []
    acc = 0
    py = []  # predicted y
    ty = []  # true y

    dec_values = (c_double * m.nr_class)()

    for i in range(learner_prob.l):
        label = liblinear.liblinear.predict_values(m, learner_prob.x[i], dec_values)
        all_dec_values += [dec_values[:m.nr_class]]
        py += [label]
        ty += [learner_prob.y[i]]

        if label == learner_prob.y[i]:
            acc += 1

    acc /= float(learner_prob.l)

    return py, acc, all_dec_values, ty



if __name__ == '__main__':
    argv = sys.argv
    if len(argv) < 2: #4 or '-v' not in argv:
        print("{0} -v fold [other liblinear_options] [learner_opts] training-data".format(argv[0]))
        sys.exit(-1)
    data_file_name = argv[-1]
    learner_opts, liblinear_opts = [], []
    i = 1
    while i < len(argv)-1:
        if argv[i] in ["-D", "-N", "-I", "-T"]:
            learner_opts += [argv[i], argv[i+1]]
            i += 2
        else :
            liblinear_opts += [argv[i]]
            i += 1
    m = train(data_file_name, learner_opts, liblinear_opts)


#!/usr/bin/env python

import os
import sys

sys.path = [os.path.dirname(os.path.abspath(__file__))] + sys.path
from liblinear import *

def svm_read_problem(data_file_name):
    """
    svm_read_problem(data_file_name) -> [y, x]

    Read LIBSVM-format data from data_file_name and return labels y
    and data instances x.
    """
    prob_y = []
    prob_x = []
    for line in open(data_file_name):
        line = line.split(None, 1)
        # In case an instance with all zero features
        if len(line) == 1: line += ['']
        label, features = line
        xi = {}
        for e in features.split():
            ind, val = e.split(":")
            xi[int(ind)] = float(val)
        prob_y += [float(label)]
        prob_x += [xi]
    return (prob_y, prob_x)

def load_model(model_file_name):
    """
    load_model(model_file_name) -> model

    Load a LIBLINEAR model from model_file_name and return.
    """
    model = liblinear.load_model(model_file_name.encode())
    if not model:
        print("can't open model file %s" % model_file_name)
        return None
    model = toPyModel(model)
    return model

def save_model(model_file_name, model):
    """
    save_model(model_file_name, model) -> None

    Save a LIBLINEAR model to the file model_file_name.
    """
    liblinear.save_model(model_file_name.encode(), model)

def evaluations(ty, pv):
    """
    evaluations(ty, pv) -> (ACC, MSE, SCC)

    Calculate accuracy, mean squared error and squared correlation coefficient
    using the true values (ty) and predicted values (pv).
    """
    if len(ty) != len(pv):
        raise ValueError("len(ty) must equal to len(pv)")
    total_correct = total_error = 0
    sumv = sumy = sumvv = sumyy = sumvy = 0
    for v, y in zip(pv, ty):
        if y == v:
            total_correct += 1
        total_error += (v-y)*(v-y)
        sumv += v
        sumy += y
        sumvv += v*v
        sumyy += y*y
        sumvy += v*y
    l = len(ty)
    ACC = 100.0*total_correct/l
    MSE = total_error/l
    try:
        SCC = ((l*sumvy-sumv*sumy)*(l*sumvy-sumv*sumy))/((l*sumvv-sumv*sumv)*(l*sumyy-sumy*sumy))
    except:
        SCC = float('nan')
    return (ACC, MSE, SCC)

def train(arg1, arg2=None, arg3=None):
    """
    train(y, x [, options]) -> model | ACC
    train(prob [, options]) -> model | ACC
    train(prob, param) -> model | ACC

    Train a model from data (y, x) or a problem prob using
    'options' or a parameter param.
    If '-v' is specified in 'options' (i.e., cross validation)
    either accuracy (ACC) or mean-squared error (MSE) is returned.

    options:
        -s type : set type of solver (default 1)
          for multi-class classification
             0 -- L2-regularized logistic regression (primal)
             1 -- L2-regularized L2-loss support vector classification (dual)
             2 -- L2-regularized L2-loss support vector classification (primal)
             3 -- L2-regularized L1-loss support vector classification (dual)
             4 -- support vector classification by Crammer and Singer
             5 -- L1-regularized L2-loss support vector classification
             6 -- L1-regularized logistic regression
             7 -- L2-regularized logistic regression (dual)
          for regression
            11 -- L2-regularized L2-loss support vector regression (primal)
            12 -- L2-regularized L2-loss support vector regression (dual)
            13 -- L2-regularized L1-loss support vector regression (dual)
        -c cost : set the parameter C (default 1)
        -p epsilon : set the epsilon in loss function of SVR (default 0.1)
        -e epsilon : set tolerance of termination criterion
            -s 0 and 2
                |f'(w)|_2 <= eps*min(pos,neg)/l*|f'(w0)|_2,
                where f is the primal function, (default 0.01)
            -s 11
                |f'(w)|_2 <= eps*|f'(w0)|_2 (default 0.001)
            -s 1, 3, 4, and 7
                Dual maximal violation <= eps; similar to liblinear (default 0.)
            -s 5 and 6
                |f'(w)|_inf <= eps*min(pos,neg)/l*|f'(w0)|_inf,
                where f is the primal function (default 0.01)
            -s 12 and 13
                |f'(alpha)|_1 <= eps |f'(alpha0)|,
                where f is the dual function (default 0.1)
        -B bias : if bias >= 0, instance x becomes [x; bias]; if < 0, no bias term added (default -1)
        -wi weight: weights adjust the parameter C of different classes (see README for details)
        -v n: n-fold cross validation mode
        -q : quiet mode (no outputs)
    """
    prob, param = None, None
    if isinstance(arg1, (list, tuple)):
        assert isinstance(arg2, (list, tuple))
        y, x, options = arg1, arg2, arg3
        prob = problem(y, x)
        param = parameter(options)
    elif isinstance(arg1, problem):
        prob = arg1
        if isinstance(arg2, parameter):
            param = arg2
        else :
            param = parameter(arg2)
    if prob == None or param == None :
        raise TypeError("Wrong types for the arguments")

    prob.set_bias(param.bias)
    liblinear.set_print_string_function(param.print_func)
    err_msg = liblinear.check_parameter(prob, param)
    if err_msg :
        raise ValueError('Error: %s' % err_msg)

    if param.cross_validation:
        l, nr_fold = prob.l, param.nr_fold
        target = (c_double * l)()
        liblinear.cross_validation(prob, param, nr_fold, target)
        ACC, MSE, SCC = evaluations(prob.y[:l], target[:l])
        if param.solver_type in [L2R_L2LOSS_SVR, L2R_L2LOSS_SVR_DUAL, L2R_L1LOSS_SVR_DUAL]:
            print("Cross Validation Mean squared error = %g" % MSE)
            print("Cross Validation Squared correlation coefficient = %g" % SCC)
            return MSE
        else:
            print("Cross Validation Accuracy = %g%%" % ACC)
            return ACC
    else :
        m = liblinear.train(prob, param)
        m = toPyModel(m)

        return m

def predict(y, x, m, options=""):
    """
    predict(y, x, m [, options]) -> (p_labels, p_acc, p_vals)

    Predict data (y, x) with the SVM model m.
    options:
        -b probability_estimates: whether to output probability estimates, 0 or 1 (default 0); currently for logistic regression only
        -q quiet mode (no outputs)

    The return tuple contains
    p_labels: a list of predicted labels
    p_acc: a tuple including  accuracy (for classification), mean-squared
           error, and squared correlation coefficient (for regression).
    p_vals: a list of decision values or probability estimates (if '-b 1'
            is specified). If k is the number of classes, for decision values,
            each element includes results of predicting k binary-class
            SVMs. if k = 2 and solver is not MCSVM_CS, only one decision value
            is returned. For probabilities, each element contains k values
            indicating the probability that the testing instance is in each class.
            Note that the order of classes here is the same as 'model.label'
            field in the model structure.
    """

    def info(s):
        print(s)

    predict_probability = 0
    argv = options.split()
    i = 0
    while i < len(argv):
        if argv[i] == '-b':
            i += 1
            predict_probability = int(argv[i])
        elif argv[i] == '-q':
            info = print_null
        else:
            raise ValueError("Wrong options")
        i+=1

    solver_type = m.param.solver_type
    nr_class = m.get_nr_class()
    nr_feature = m.get_nr_feature()
    is_prob_model = m.is_probability_model()
    bias = m.bias
    if bias >= 0:
        biasterm = feature_node(nr_feature+1, bias)
    else:
        biasterm = feature_node(-1, bias)
    pred_labels = []
    pred_values = []

    if predict_probability:
        if not is_prob_model:
            raise TypeError('probability output is only supported for logistic regression')
        prob_estimates = (c_double * nr_class)()
        for xi in x:
            xi, idx = gen_feature_nodearray(xi, feature_max=nr_feature)
            xi[-2] = biasterm
            label = liblinear.predict_probability(m, xi, prob_estimates)
            values = prob_estimates[:nr_class]
            pred_labels += [label]
            pred_values += [values]
    else:
        if nr_class <= 2:
            nr_classifier = 1
        else:
            nr_classifier = nr_class
        dec_values = (c_double * nr_classifier)()
        for xi in x:
            xi, idx = gen_feature_nodearray(xi, feature_max=nr_feature)
            xi[-2] = biasterm
            label = liblinear.predict_values(m, xi, dec_values)
            values = dec_values[:nr_classifier]
            pred_labels += [label]
            pred_values += [values]
    if len(y) == 0:
        y = [0] * len(x)
    ACC, MSE, SCC = evaluations(y, pred_labels)
    l = len(y)
    if solver_type in [L2R_L2LOSS_SVR, L2R_L2LOSS_SVR_DUAL, L2R_L1LOSS_SVR_DUAL]:
        info("Mean squared error = %g (regression)" % MSE)
        info("Squared correlation coefficient = %g (regression)" % SCC)
    else:
        info("Accuracy = %g%% (%d/%d) (classification)" % (ACC, int(l*ACC/100), l))

    return pred_labels, (ACC, MSE, SCC), pred_values



# coding: utf-8

import os

from setuptools.command.install import install

from setuptools import setup


with open('README.rst') as f:
    LONG_DESCRIPTION = f.read()


class MakeCommand(install):
    def run(self):
        os.system('make')
        common_dir = 'tgrocery/learner'
        target_dir = '%s/%s' % (self.build_lib, common_dir)
        self.mkpath(target_dir)
        os.system('cp %s/util.so.1 %s' % (common_dir, target_dir))
        common_dir = 'tgrocery/learner/liblinear'
        target_dir = '%s/%s' % (self.build_lib, common_dir)
        self.mkpath(target_dir)
        os.system('cp %s/liblinear.so.1 %s' % (common_dir, target_dir))
        install.run(self)


setup(
    name='tgrocery',
    version='0.1.4',
    packages=['tgrocery', 'tgrocery.learner', 'tgrocery.learner.liblinear.python'],
    url='https://github.com/2shou/TextGrocery',
    license='BSD',
    author='2shou',
    author_email='gavin.zgz@gmail.com',
    description='A simple short-text classification tool based on LibLinear',
    long_description=LONG_DESCRIPTION,
    install_requires=['jieba'],
    keywords='text classification svm liblinear libshorttext',
    cmdclass={'install': MakeCommand}
)
